@@ -1,6 +1,7 @@
 #
 # This file is licensed under the Affero General Public License (AGPL) version 3.
 #
+# Copyright 2014 - 2016 OpenMarket Ltd
 # Copyright (C) 2023 New Vector, Ltd
 #
 # This program is free software: you can redistribute it and/or modify
@@ -468,8 +469,7 @@ class Notifier:
         new_token: RoomStreamToken,
         users: Optional[Collection[Union[str, UserID]]] = None,
         rooms: Optional[StrCollection] = None,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @overload
     def on_new_event(
@@ -478,8 +478,7 @@ class Notifier:
         new_token: MultiWriterStreamToken,
         users: Optional[Collection[Union[str, UserID]]] = None,
         rooms: Optional[StrCollection] = None,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @overload
     def on_new_event(
@@ -496,8 +495,7 @@ class Notifier:
         new_token: int,
         users: Optional[Collection[Union[str, UserID]]] = None,
         rooms: Optional[StrCollection] = None,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     def on_new_event(
         self,
